%--------------------------------------------------------------------------
%   Usage: crediable_region_est_ub_main;
%
%   Crediable region estimation from computing an upper bound by 
%           gamma_a := g(x') + N*(tau_a + 1)
%   where x' is the solution of mim g(x), N is the size of x, 
%   tau_a = sqrt(16*log(3/a)/N), and a \in (4exp(-N/3), 1).
% 
%   Other operators: A is the forward measurement operator and At the 
%   associated adjoint operator, Psit is wavelet transform and Psi its 
%   adjoint, and delta is the parameter balancing the L1 and L2 terms.
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai
%   Last updated: 01/03/2017
%--------------------------------------------------------------------------

%% Define paths
addpath ./cr_fun

%% Clear workspace
clc; clear; close all;

%% data name
stringname = {'M31', 'CYN', 'W28', '3c288'};

for i=2:4
    % path to save generated data
    pathData_ana=['../data/', stringname{i}, '_mat_ana/'];
    pathData_syn=['../data/', stringname{i}, '_mat_syn/'];
    pathResult=['../result/',stringname{i},'/'];
    if 0 == exist(pathResult)
        mkdir(pathResult);
    end
    
    %% load image 
    load([pathData_syn, stringname{i}, '_operators.mat']);
    load([pathData_syn, stringname{i}, '_measure_noise.mat']);
    load([pathData_syn, stringname{i}, '_measure_org.mat']);
    load([pathData_syn, stringname{i}, '_data_org.mat']);
    load([pathData_syn, stringname{i}, '_synthesis_result.mat']);
    sol_syn=sol;
    load([pathData_ana, stringname{i}, '_analysis_result.mat']);
    sol_ana=sol;
    num_ind = size(Ma,1);
    N = numel(im(:));

    %% Define operators
    % Set the DWT mode to periodization
    dwtmode('per');

    % Decomposition level of the wavelet transform.
    nlevel = 4;

    % Daubechies 8 wavelet operator
    [C,S] = wavedec2(im,nlevel,'db8');
    Psit = @(x) wavedec2(x, nlevel, 'db8');
    Psi = @(x) waverec2(x, S, 'db8');

%     % Composition (Masking o Fourier)
%     A = @(x) Ma*reshape(fft2(x)/sqrt(num_ind), numel(x), 1);
%     At = @(x) (ifft2(reshape(Ma'*x(:), size(im))*sqrt(num_ind)));

    % Energy (cost) function
    delta = 1e4; % regularisation parameter
    energy_ana = @(sol) delta*sum(sum(abs(Psit(sol)))) ...
        + norm(y - A(sol),'fro')^2/(2*sigma_noise^2);
    energy_syn = @(sol) delta*sum(sum(abs(sol))) ...
        + norm(y - A(Psi(sol)),'fro')^2/(2*sigma_noise^2);
        
    %% compute credible region - upper bound
    % compute upper bound using analysis model
    energy_ana_u = energy_ana(sol_ana);
    j=1;
    for alpha = 0.01:0.01:0.99
        gamma_alpha_ana(j) = energy_ana_u + N*(sqrt(16*log(3/(1-alpha))/N) + 1);
        j = j + 1;
    end
    
    % compute upper bound usig synthesis model to compare
    j=1;
    sol_img_syn_t=Psi(sol_syn);sol_img_syn_t(sol_img_syn_t<0)=0;
    energy_syn_u = energy_ana(sol_img_syn_t);
    for alpha = 0.01:0.01:0.99
        gamma_alpha_syn(j) = energy_syn_u + N*(sqrt(16*log(3/(1-alpha))/N) + 1);
        j = j + 1;
    end
    
    save([pathResult, stringname{i}, '_cr_ub.mat'], 'gamma_alpha_ana', 'gamma_alpha_syn');
    
    %% error estimation
    [xLen,yLen] = size(im);
    % use bound from analysis model
    gamma_alpha_ana_sel = max(gamma_alpha_ana);
    gamma_alpha_syn_sel = max(gamma_alpha_syn);
       
    % --- find credible intervals for grid image  
    gridSize = [10, 20, 30];
    stepSize=0.1./gridSize;
    [gridImg_lh_ana, gridImg2_lh_ana] = FindItv4GridImg(gridSize,sol_ana, ...
        stepSize, energy_ana, gamma_alpha_ana_sel);
%     [gridImg_lh_syn, gridImg2_lh_syn] = FindItv4GridImg(gridSize, ...
%         sol_img_syn_t,stepSize, energy_ana, gamma_alpha_syn_sel);

    gridImg_lh_syn = 0;
    save([pathResult, stringname{i}, '_cr_lc', '.mat'], 'gridImg_lh_ana', 'gridImg_lh_syn');
    % ---
  
        
    %% show given data    
    % show original (clean) image
    if i==2
        figure, imagesc(log10(im),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
    else
        figure, imagesc(log10(im),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    end
    colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))
    fname = [pathResult, stringname{i}, '_given_img'  ,'.png'] ;
    print('-r200', '-dpng', fname)

    % show dirty image - by inverse fft directly
    RSNRd = 20*log10(norm(im,'fro')/norm(im-abs(At(y)),'fro'));
    if i==2
        figure, imagesc(log10(abs(At(y))),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    else
        figure, imagesc(log10(abs(At(y))),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    end
    colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)),    
    title(['SNR=',num2str(RSNRd), 'dB']); truesize(size(im))
    fname = [pathResult, stringname{i}, '_dirty_img'  ,'.png'] ;
    print('-r200', '-dpng', fname)
    
    % show mask used
    mask = ifftshift(reshape(full(sum(Ma,1)),size(im)));
    figure,imshow(mask,[],'border','tight');
    fname = [pathResult, stringname{i}, '_mask'  ,'.png'] ;
    print('-r200', '-dpng', fname)
    
    % show given y - no noise
    y2_org = ifftshift(reshape(Ma'*y_org, size(im)));
    figure,imagesc(abs(y2_org)),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
    truesize(size(im)), 
    colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
    fname = [pathResult, stringname{i}, '_meaure_org'  ,'.png'] ;
    print('-r200', '-dpng', fname)
    
    % show given y - with noise
    y2_noise = ifftshift(reshape(Ma'*y, size(im)));
    figure,imagesc(abs(y2_noise)),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    truesize(size(im)),
    colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
    fname = [pathResult, stringname{i}, '_meaure_noise'  ,'.png'] ;
    print('-r200', '-dpng', fname)
    
    % show result by using analysis model
    RSNRa = 20*log10(norm(im,'fro')/norm(im-sol_ana,'fro'));
    if i==2
        figure, imagesc(log10(sol_ana),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
    else
        figure, imagesc(log10(sol_ana),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    end
    colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)),
    title(['SNR=',num2str(RSNRa), 'dB']); truesize(size(im))
    fname = [pathResult, stringname{i}, '_result_ana'  ,'.png'] ;
    print('-r200', '-dpng', fname)
    
    % show result by using synthesis model
    sol_img_syn = Psi(sol_syn); sol_img_syn(sol_img_syn<0)=0;
    RSNRs = 20*log10(norm(im,'fro')/norm(im-sol_img_syn,'fro'));
    if i==2
        figure, imagesc(log10(sol_img_syn),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    else
        figure, imagesc(log10(sol_img_syn),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
    end
    title(['SNR=',num2str(RSNRs), 'dB']); truesize(size(im)),
    colormap(CubeHelix(256,0.5,-1.5,1.2,1.0));
    fname = [pathResult, stringname{i}, '_result_syn'  ,'.png'] ;
    print('-r200', '-dpng', fname)
    
    % show computed credible region upper bound
    alpha = 0.01:0.01:0.99;
    figure, plot(alpha, gamma_alpha_ana, '-r', alpha, gamma_alpha_syn, '-b');
    legend({'MAP-ana', 'MAP-syn'}, 'FontSize',18); set(gca,'fontsize',20)
    fname = [pathResult, stringname{i}, '_cr_gamma_alpha'  ,'.png'];
    print('-r200', '-dpng', fname)
    
    % show credible area for image accroding given grid
    for j=1:length(gridSize)  
        % choose scale for image showing
        if i==1
            scaleL=0; scaleH=0.3;
        elseif i==2
            scaleL=0; scaleH=0.1;          
        elseif i==3
            scaleL=0; scaleH=0.2;
        else 
            scaleL=0; scaleH=0.2;    
        end
           
        % for analysis result
        figure, imagesc(gridImg_lh_ana{j}.cr_bound_h - gridImg_lh_ana{j}.cr_bound_l,[scaleL,scaleH]);  
        axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
        truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
        fname = [pathResult, stringname{i}, '_cr_ana_grid_', num2str(gridSize(j))  ,'.png'];
        print('-r200', '-dpng', fname)
        
%         figure, imagesc(gridImg2_lh_ana{j}.cr_bound_h - gridImg2_lh_ana{j}.cr_bound_l);  
%         axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%         truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%         fname = [pathResult, stringname{i}, '_cr_ana_2_grid_', num2str(gridSize(j))  ,'.png'];
%         print('-r200', '-dpng', fname)
        
%         % for synthesis result
%         figure, imagesc(gridImg_lh_syn{j}.cr_bound_h - gridImg_lh_syn{j}.cr_bound_l,[scaleL,scaleH]); 
%         axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%         truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%         fname = [pathResult, stringname{i}, '_cr_syn_grid_', num2str(gridSize(j))  ,'.png'];
%         print('-r200', '-dpng', fname)

%         figure, imagesc(gridImg2_lh_syn{j}.cr_bound_h - gridImg2_lh_syn{j}.cr_bound_l); 
%         axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
%         truesize(size(im)), colormap(CubeHelix(256,0.5,-1.5,1.2,1.0))
%         fname = [pathResult, stringname{i}, '_cr_syn_2_grid_', num2str(gridSize(j))  ,'.png'];
%         print('-r200', '-dpng', fname)
    end
    

    
end




















