function pmcmc_hypothesis_test(stringname,imgNum,nameMethod,delta)
%--------------------------------------------------------------------------
%   Usage: crediable_region_hpt_test_main;
%
%   Hypothesis testing by removing features in the MAP estimate.
% 
%   Firstly, compute the upper bound of the credible region, i.e. 
%           gamma_a := g(x') + N*(tau_a + 1)
%   where x' is the solution of mim g(x), N is the size of x, 
%   tau_a = sqrt(16*log(3/a)/N), and a \in (4exp(-N/3), 1).
% 
%   Secondly, generate a surrogate surrogate image of the MAP estimate.
% 
%   Thirdly, compare the energy with the upper bound to evaluate given area
% 
%   Other operators: A is the forward measurement operator and At the 
%   associated adjoint operator, Psit is wavelet transform and Psi its 
%   adjoint, and delta is the parameter balancing the L1 and L2 terms.
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai
%   Last updated: 01/03/2017
%--------------------------------------------------------------------------

%% Define paths
addpath ./cr_fun

%% 
% path to save generated data
pathData_ana=['../data/', stringname, '_mat_ana/'];
pathData_syn=['../data/', stringname, '_mat_syn/'];
pathResult=['../result/',stringname,'/'];
if 0 == exist(pathResult)
    mkdir(pathResult);
end

%% load image 
load([pathData_syn, stringname, '_operators.mat']);
load([pathData_syn, stringname, '_measure_noise.mat']);
load([pathData_syn, stringname, '_measure_org.mat']);
load([pathData_syn, stringname, '_data_org.mat']);
load([pathData_syn, stringname, '_', nameMethod, '_hpd_cr.mat']);
load([pathData_syn, stringname, '_', nameMethod, '_sample_mean.mat']);
load([pathData_syn, stringname, '_samples_', nameMethod,'_syn.mat']); 
load([pathData_ana, stringname, '_samples_', nameMethod,'_ana.mat']);

nameMethodTemp = nameMethod;
nameMethodTemp(2:end) = lower(nameMethodTemp(2:end));
samAna = eval(['sam',nameMethodTemp,'Ana']);
samSyn = eval(['sam',nameMethodTemp,'Syn']);

%% Define operators
% Set the DWT mode to periodization
dwtmode('per');

% Decomposition level of the wavelet transform.
nlevel = 4;

% Daubechies 8 wavelet operator
[C,S] = wavedec2(im,nlevel,'db8');
Psit = @(x) wavedec2(x, nlevel, 'db8');
Psi = @(x) waverec2(x, S, 'db8');

% Energy (cost) function
energy_ana = @(sol) delta*sum(sum(abs(Psit(sol)))) ...
    + norm(y - A(sol),'fro')^2/(2*sigma_noise^2);
energy_syn = @(sol) delta*sum(sum(abs(sol))) ...
    + norm(y - A(Psi(sol)),'fro')^2/(2*sigma_noise^2);

%%
% % use the mean value for hypothesis test
% sol_syn=meanXtrace_syn;
% sol_ana=meanXtrace_ana;

% use one sample for hypothesis test
samNum = size(samSyn.Xtrace,1);
sampleIdx = floor(samNum/2);
sol_syn=Psi(squeeze(samSyn.Xtrace(sampleIdx,:,:)));
sol_ana=squeeze(samAna.Xtrace(sampleIdx,:,:));

%% Generate surrogate image - inpainting
% generate mask 
maskTest = MaskGet(im, stringname);
% inpainting 
sol_inp_syn = ImgInp(Psit, Psi, sol_syn, maskTest);
sol_inp_ana = ImgInp(Psit, Psi, sol_ana, maskTest);

save([pathResult, stringname, '_', nameMethod, '_sol_inp.mat'], 'sol_inp_syn', 'sol_inp_ana');


%% Hypothesis testing
% obtain hpd credible region
gamma_alpha_ana = gamma_alpha_hpd_cr_ana;
gamma_alpha_syn = gamma_alpha_hpd_cr_syn;

% use bound from analysis model
gamma_alpha_ana_min = min(gamma_alpha_ana);
gamma_alpha_syn_min = min(gamma_alpha_syn);

% check whether a given area is removable     
areaRmv_ana = AreaRmvCheck(sol_ana, sol_inp_ana, maskTest, energy_ana, ...
    gamma_alpha_ana_min);
areaRmv_syn = AreaRmvCheck(sol_syn, sol_inp_syn, maskTest, energy_ana, ...
    gamma_alpha_syn_min);

save([pathResult, stringname, '_', nameMethod, '_area_rmv.mat'], 'areaRmv_ana', 'areaRmv_syn');


%% show given data    
% show inpainting result by using synthesis model
sol_inp_syn(sol_inp_syn<0)=0;
if imgNum==2
    figure, imagesc(log10(sol_inp_syn),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
else
    figure, imagesc(log10(sol_inp_syn),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
end
colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))
fname = [pathResult, stringname, '_', nameMethod, '_sol_inp_syn'  ,'.png'] ;
print('-r200', '-dpng', fname)

% show inpainting result by using analysis model
sol_inp_ana(sol_inp_ana<0)=0;
if imgNum==2
    figure, imagesc(log10(sol_inp_ana),[-3 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]);
else
    figure, imagesc(log10(sol_inp_ana),[-2 0]),axis off; cb = colorbar('FontSize',18); a=get(cb); a =  a.Position; set(cb,'Position',[a(1)+0.055 a(2)-0.004 0.015 0.78]); 
end
colormap(CubeHelix(256,0.5,-1.5,1.2,1.0)), truesize(size(im))
fname = [pathResult, stringname, '_', nameMethod, '_sol_inp_ana'  ,'.png'] ;
print('-r200', '-dpng', fname)

% show computed credible region upper bound and the energy of surrogate
alpha = 0.01:0.01:0.99;
figure, 
p1 = plot(alpha(1:5:end), gamma_alpha_ana(1:5:end), '-r');
hold on
p2 = plot(alpha(1:5:end), gamma_alpha_syn(1:5:end), '-b');
lineType_ana = {'--rd', '--rs', '--r+', '--r*', '--ro'};
lineType_syn = {'--bd', '--bs', '--b+', '--b*', '--bo'};
p3=p2; p4=p2; p5=p2; p6=p2; p7=p2; p8=p2; p9=p2; p10=p2; p11=p2; p12=p2;
pType = {p1, p2, p3, p4, p5, p6, p7, p8, p9, ...
    p10, p11, p12};
for j=1:length(areaRmv_ana)
    pType{2*j+1} = plot(alpha(1:5:end), ones(1,length(gamma_alpha_ana(1:5:end))).*areaRmv_ana{j}.engSgt, lineType_ana{j});
    pType{2*j+2} = plot(alpha(1:5:end), ones(1,length(gamma_alpha_syn(1:5:end))).*areaRmv_syn{j}.engSgt, lineType_syn{j});
    disp(['r_max_ana: ', num2str(max(gamma_alpha_ana(:))), '!  '...
    'r_max_syn: ', num2str(max(gamma_alpha_syn(:))), '!  '...
    'r_min_ana: ', num2str(min(gamma_alpha_ana(:))), '!  '...
    'r_min_syn: ', num2str(min(gamma_alpha_syn(:))), '!  '...
    'inp_ana: ', num2str(areaRmv_ana{j}.engSgt), '!  '...
    'inp_syn: ', num2str(areaRmv_syn{j}.engSgt), '!  ']);
end  
legendType = {'ana: ub', 'syn: ub', 'ana: sgt1', 'syn: sgt1', ...
    'ana: sgt2', 'syn: sgt2', 'ana: sgt3', 'syn: sgt3', ...
    'ana: sgt4', 'syn: sgt4', 'ana: sgt5', 'syn: sgt5' };
legend([pType{1:2+2*length(areaRmv_ana)}], ...
    legendType{1:2+2*length(areaRmv_ana)}, 'FontSize',15);
hold off

fname = [pathResult, stringname, '_', nameMethod, '_hp_area_rmv'  ,'.png'] ;
print('-r200', '-dpng', fname)  





















