function sol = sopt_fb_uca(y, delta, A, At, Psi, Psit, param)
%--------------------------------------------------------------------------
%   Usage: sol = sopt_fb_uca(y, delta, A, At, Psi, Psit, param);
%
%   Solve the unconstrained Basis Pursuit Denoising (BPDN) problem, the 
%   analysis form, i.e.
%           min delta*||Psit x||_1 + ||y-A x||_2
%   where y contains the measurements, A is the forward measurement operator 
%   and At the associated adjoint operator, Psit is wavelet transform and
%   Psi its adjoint, and delta is the parameter balancing the L1 and L2 terms.
%   sol is solution of the proposed problem. 
%   The structure param should contain the following fields:
%
%   Parameters:
% 
%   - verbose: Verbosity level (0 = no log, 1 = summary at convergence, 
%       2 = print main steps; default = 1).
%
%   - max_iter: Maximum number of iterations (default = 200).
%
%   - rel_obj: Minimum relative change of the objective value 
%       (default = 1e-4).  The algorithm stops if
%           | ||x(t)||_1 - ||x(t-1)||_1 | / ||x(t)||_1 < rel_obj,
%       where x(t) is the estimate of the solution at iteration t.
% 
%   Proximal L1 operator:
%
%   - rel_obj_L1: Used as stopping criterion for the proximal L1
%       operator. Minimum relative change of the objective value between
%       two successive estimates.
%
%   - max_iter_L1: Used as stopping criterion for the proximal L1
%       operator. Maximun number of iterations.
% 
%   - nu_L1: Bound on the norm^2 of the operator Psi, i.e.
%       ||Psi x||^2 <= nu * ||x||^2 (default = 1).
% 
%   - tight_L1: 1 if Psit is a tight frame, 0 otherwise 
%       (default = 1).
% 
%   - weights: Weights for a weighted L1-norm defined
%       by sum_i{weights_i.*abs(x_i)} (default = 1). 
%
%   - pos_l1: Positivity flag (1 to impose positivity, 0 otherwise;
%       default = 0).
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai & Christopher Wallis
%   Last updated: 14/06/2016
%--------------------------------------------------------------------------

% Optional input arguments
if ~isfield(param, 'verbose'), param.verbose = 1; end
if ~isfield(param, 'rel_obj'), param.rel_obj = 1e-4; end
if ~isfield(param, 'max_iter'), param.max_iter = 200; end
if ~isfield(param, 'pos_l1'), param.pos_l1 = 0; end
if ~isfield(param, 'gamma'), param.gamma = 1e-3; end

% Input arguments for prox L1
param_L1.Psi = Psi; param_L1.Psit = Psit; param_L1.pos = param.pos_l1;
param_L1.verbose = param.verbose; param_L1.rel_obj = param.rel_obj;
if isfield(param, 'nu_L1')
    param_L1.nu = param.nu_L1;
end
if isfield(param, 'tight_L1')
    param_L1.tight = param.tight_L1;
end
if isfield(param, 'rel_obj_L1')
    param_L1.rel_obj = param.rel_obj_L1;
end
if isfield(param, 'weights')
    param_L1.weights = param.weights;
else
    param_L1.weights = 1;
end

% Initialization
if isfield(param,'initsol')
    sol = param.initsol;
else
    sol = abs(At(y)); 
end


%% start iteration
for iter=1:param.max_iter   
    if param.verbose >= 1
        fprintf('Iteration %i:\n', iter);
    end
    
    sol0=sol;
    
    % Froward iteraion
    dummy = sol - 2*param.gamma*At(A(sol) - y);
%     dummy = abs(dummy);
    % Backward - proximal L1 operator
    [sol,~] = sopt_prox_L1(dummy, delta*param.gamma, param_L1);
%     sol=abs(sol);
    sol=real(sol);
    
    % Global stopping criterion  
    rel_norm = sum(sum(abs(sol - sol0)))/sum(sum(abs(sol)));
    energy(iter) = delta*sum(sum(abs(Psit(sol)))) + norm(y - A(sol),'fro')^2;
    if param.verbose >= 1
        fprintf(' energy = %i,\t rel_norm = %e\n', energy(iter), rel_norm);
    end
    if (rel_norm < param.rel_obj) || (iter >= param.max_iter)
        break;
    end
end

sol(sol<0) = 0;


