function sol = sopt_fb_ucs(y, delta, A, At, Psi, Psit, param)
%--------------------------------------------------------------------------
%   Usage: sol = sopt_fb_ucs(y, delta, A, At, Psi, Psit, param);
%
%   Solve the unconstrained Basis Pursuit Denoising (BPDN) problem, the 
%   analysis form, i.e.
%           min delta*||a||_1 + ||y - A psi a||_2
%   where y contains the measurements, A is the forward measurement operator 
%   and At the associated adjoint operator, Psit is wavelet transform and
%   Psi its adjoint, and delta is the parameter balancing the L1 and L2 terms.
%   sol is solution of the proposed problem. 
%   The structure param should contain the following fields:
%
%   Parameters:
% 
%   - verbose: Verbosity level (0 = no log, 1 = summary at convergence, 
%       2 = print main steps; default = 1).
%
%   - max_iter: Maximum number of iterations (default = 200).
%
%   - rel_obj: Minimum relative change of the objective value 
%       (default = 1e-4).  The algorithm stops if
%           ||a(t) - a(t-1)||_1 / ||a(t)||_1 < rel_obj,
%       where a(t) is the estimate of the solution at iteration t.
%
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai & Christopher Wallis
%   Last updated: 14/06/2016
%--------------------------------------------------------------------------

% Optional input arguments
if ~isfield(param, 'verbose'), param.verbose = 1; end
if ~isfield(param, 'rel_obj'), param.rel_obj = 1e-4; end
if ~isfield(param, 'max_iter'), param.max_iter = 200; end
if ~isfield(param, 'gamma'), param.gamma = 1e-3; end

soft = @(z, T) sign(z).*max(abs(z)-T, 0);

% Initialization
if isfield(param,'initsol')
    sol = param.initsol;
else
    sol = Psit(abs(At(y))); 
end


%% start iteration
for iter=1:param.max_iter   
    if param.verbose >= 1
        fprintf('Iteration %i:\n', iter);
    end
    
    sol0=sol;
    
    % Froward iteraion
    dummy = sol - 2*param.gamma*Psit(At(A(Psi(sol)) - y));
    % Backward - proximal L1 operator
    sol= soft(dummy, delta*param.gamma);
    sol=real(sol);
    
    % Global stopping criterion  
    rel_norm = sum(sum(abs(sol - sol0)))/sum(sum(abs(sol)));
    energy(iter) = delta*sum(sum(abs(sol))) + norm(y - A(Psi(sol)),'fro')^2;
    if param.verbose >= 1
        fprintf(' energy = %i,\t rel_norm = %e\n', energy(iter), rel_norm);
    end
    if (rel_norm < param.rel_obj) || (iter >= param.max_iter)
        break;
    end
end

% sol(sol<0) = 0;


