function [Xtrace,logPiTrace,time] = MYULA(X0,logPi,proxH,gradF,options)
%% This MATLAB function implements the forward-backward Metropolis adjusted Langevin algorithm described in 
%
% [1] M. Pereyra, "Proximal Markov chain Monte Carlo algorithms," J. Comp. Graph. Statist., submitted. 
% E-print available on Arxiv: http://arxiv.org/abs/1306.0187
%
%
% Inputs
%
%   - X0        :   initial state of the Markov chain
%   - logPi     :   function handler of type @(X) to evaluate \log\pi(X) up an additive constant
%   - proxH     :   function handler of type @(X,lambda) to evaluate the lambda-proximity mapping of h at X
%   - gradF     :   function handler of type @(X) to evaluate the gradient of logf at X
%
%   - options (stucture)
%       -options.samples    : number of sampler to be generated
%       -options.thinning   : thinning factor (i.e., number of MCMC iterations
%                             per sample)~
%       -options.lambda     : value of lambda
%
% Outputs
%
%   - Xtrace        :   trace of X
%   - logPiTrace    :   trace of \log\pi
%   - acceptTrace   :   record indicating which iterations where accepted
%
% See example2.m for an example of how to use this function.
%
% by Marcelo Pereyra, http://www.stats.bris.ac.uk/~mp12320/
% Copyright (c) 2013 Marcelo Pereyra
% 
%   Modified by: Xiaohao Cai
%   Last updated: 29/11/2016

if(~isfield(options,'displayProgress'))
    options.displayProgress = false;
end

    
%% Simulation setup
% Allocate chains
n = size(X0);
Xtrace = zeros([options.samples n]);
% logPiTrace(options.samples*options.thinning)=0;
logPiTrace(options.samples)=0;

% Check if X is a vector or a matrix and choose the corresponding norm
if(n(2)==1)
    p = 2;      % L2 norm
else
    p = 'fro';  % Frobenius norm
end

% Use backward-forward splitting to approximate proxPi using proxH and gradF 
drift = @(X,lambda,mu) real((1-lambda/mu)*X + lambda*proxH(X,mu)/mu + ...
    lambda*gradF(X)/2); 

% Initialize variables
lambda = options.lambda;            % lambda controls the proposal variance, the step-length and Moreau approximation
mu = 3*lambda;                   % mu \in [2*lambda, 5*lambda]   
Xcur = X0;                          % set initial state as current state
logPiTrace(1) = logPi(Xcur);        % record \log\pi(Xcur)
if(n(2)==1)
    Xtrace(1,:) = Xcur;     % X is vector;
else    
    Xtrace(1,:,:) = Xcur;   % X is matrix
end    

%% Run simulation
tic;
j=1;
for ii = 2:(options.samples*options.thinning + options.burningNums)
    
    % P-ULA (THIS IS THE MARKOV CHAIN)
    Xcur = drift(Xcur,lambda,mu) + sqrt(2*lambda)*randn(n);
       
    % Save sample (with thinning)
    if (ii>options.burningNums) && (~mod(ii - options.burningNums, options.thinning))
        if(n(2)==1)
            Xtrace(j,:) = Xcur;    % X is vector
            
            % Record the value of logPi
            logPiTrace(j) = logPi(Xcur);
        else
            Xtrace(j,:,:) = Xcur;  % X is matrix
            
            % Record the value of logPi
            logPiTrace(j) = logPi(Xcur);      
        end
        j = j + 1;
    end
end

% Output simulation time
time = toc;