function [Xtrace,logPiTrace,acceptTrace,muTrace,time] = PMALA(X0,logPi,proxH,gradF,options)
%% This MATLAB function implements the forward-backward Metropolis adjusted Langevin algorithm described in 
%
% [1] M. Pereyra, "Proximal Markov chain Monte Carlo algorithms," J. Comp. Graph. Statist., submitted. 
% E-print available on Arxiv: http://arxiv.org/abs/1306.0187
%
%
% Inputs
%
%   - X0        :   initial state of the Markov chain
%   - logPi     :   function handler of type @(X) to evaluate \log\pi(X) up an additive constant
%   - proxH     :   function handler of type @(X,lambda) to evaluate the lambda-proximity mapping of h at X
%   - gradF     :   function handler of type @(X) to evaluate the gradient of logf at X
%
%   - options (stucture)
%       -options.samples    : number of sampler to be generated
%       -options.thinning   : thinning factor (i.e., number of MCMC iterations
%                             per sample)~
%       -options.lambda     : value of lambda
%
% Outputs
%
%   - Xtrace        :   trace of X
%   - logPiTrace    :   trace of \log\pi
%   - acceptTrace   :   record indicating which iterations where accepted
%
% See example2.m for an example of how to use this function.
%
% by Marcelo Pereyra, http://www.stats.bris.ac.uk/~mp12320/
% Copyright (c) 2013 Marcelo Pereyra
% 
%   Modified by: Xiaohao Cai
%   Last updated: 01/03/2017

if(~isfield(options,'displayProgress'))
    options.displayProgress = false;
end

    
%% Simulation setup
% Allocate chains
n = size(X0);
Xtrace = zeros([options.samples n]);
% logPiTrace(options.samples*options.thinning + options.burningNums)=0;
logPiTrace(options.samples)=0;
acceptTrace(options.samples*options.thinning + options.burningNums)=0;
muTrace(options.samples*options.thinning + options.burningNums)=0;

% Check if X is a vector or a matrix and choose the corresponding norm
if(n(2)==1)
    p = 2;      % L2 norm
else
    p = 'fro';  % Frobenius norm
end

% Use backward-forward splitting to approximate proxPi using proxH and gradF 
drift = @(X,lambda,mu) real(X + mu*(proxH(X,lambda)-X)/lambda + mu*gradF(X));

% Initialize variables
lambda = options.lambda;            % This should be roughly 1/L_f (L_f is the Lipchitz constant of gradF).
mu = lambda/4;                      % initial value (then adjusted adaptively)
Xcur = X0;                          % set initial state as current state
logPiTrace(1) = logPi(Xcur);        % record \log\pi(Xcur)
if(n(2)==1)
    Xtrace(1,:) = Xcur;     % X is vector;
else    
    Xtrace(1,:,:) = Xcur;   % X is matrix
end    

%% Run simulation
tic;
j=1;

% Do 10 iterations of PULA to warm-start PMALA
for ii = 1:10
    driftXcur = drift(Xcur,lambda,mu);
    Xcur = driftXcur + sqrt(2*mu)*randn(n);
end

driftXcur = drift(Xcur,lambda,mu);
logPiCur = logPi(Xcur);

for ii = 2:(options.samples*options.thinning + options.burningNums)
    
    % P-ULA proposal step    
    Xnew = driftXcur + sqrt(2*mu)*randn(n);
    driftXnew = drift(Xnew,lambda,mu);
    logPiNew = logPi(Xnew);
    
    % Metropolis-Hastings step
    logPiRatio = logPiNew-logPiCur;
    logPropRatio = (norm(Xnew - driftXcur,'fro')^2 - norm(Xcur - driftXnew,'fro')^2)/(4*mu);
    
    if (log(rand(1)) < ( logPiRatio + logPropRatio))
        Xcur = Xnew;                % set candidate as new current
        driftXcur = driftXnew;      % record drift(Xnew) as current for next iteration
        logPiCur = logPiNew;
        acceptTrace(ii)=1;          % record that the candidate was accepted        
    end
    
%     % Record logPi at each iteration
%     logPiTrace(ii) = logPiCur;
    
    % Adaptation of mu to keep acceptance rate close to 50%
    mu = mu*(1+ (acceptTrace(ii)- 0.5)/ii^0.75);     % Stochastic gradient step to adjust mu
    mu = min(max(mu,lambda/1e8),lambda/2);     % Bound mu for stability
    muTrace(ii) = mu;                           % Keep track of mu
       
    % Save sample (with thinning)
    if (ii>options.burningNums) && (~mod(ii - options.burningNums, options.thinning))
        if(n(2)==1)
            Xtrace(j,:) = Xcur;    % X is vector
            
            % Record the value of logPi
            logPiTrace(j) = logPi(Xcur);
        else
            Xtrace(j,:,:) = Xcur;  % X is matrix
            
            % Record the value of logPi
            logPiTrace(j) = logPi(Xcur);
        end
        j = j + 1;
    end
      
end

% Output simulation time
time = toc;