%--------------------------------------------------------------------------
% Generate Samples bys Proximal Markov chain Monte Carlo
% This code demonstrates the P-MALA algorithm, Proximal Unadjusted Langevin 
% Algorihm (P-ULA) with Matropolis-hasting step,
% of M. Pereyra, "Proximal Markov chain Monte Carlo algorithms," Statistics
% and Computing, 2015.
% 
% Set the objective function as:
%                   G(x) = H(x) + F(X),
% where H(x) is l_1 term, and F(x) is data fidelity term
% 
%   Modified by: Xiaohao Cai
%   Last updated: 01/03/2017
%--------------------------------------------------------------------------

%% Clear workspace and command window
clc; clear; close all;

addpath('../data')
addpath('./utils');

%% data name
stringname = {'M31', 'CYN', 'W28', '3c288'};

for i=1:4
    disp(['Processing data', stringname{i}, '!']);
    
    % path to save generated data
    pathData_syn=['../data/', stringname{i}, '_mat_syn/'];
    pathData_ana=['../data/', stringname{i}, '_mat_ana/'];
    
    %% load image 
    load([pathData_syn, stringname{i}, '_operators.mat']);
    load([pathData_syn, stringname{i}, '_measure_noise.mat']);
    load([pathData_syn, stringname{i}, '_measure_org.mat']);
    load([pathData_syn, stringname{i}, '_data_org.mat']);
    num_ind = size(Ma,1);

    %% Define functions related to Bayesian model
    % Set the DWT mode to periodization
    dwtmode('per');
    % Decomposition level of the wavelet transform.
    nlevel = 4;
    % Daubechies 8 wavelet operator
    [C,S] = wavedec2(im,nlevel,'db8');
    Psit = @(x) wavedec2(x, nlevel, 'db8');
    Psi = @(x) waverec2(x, S, 'db8');
    
    % Regularisation parameter
    delta = 1e4;  % Tune to meaningful regularization parameter
    % Cost function
    logPi_ana = @(sol) - delta*sum(sum(abs(Psit(sol))))...
        - norm(y-A(sol),'fro')^2/(2*sigma_noise^2);
    logPi_syn = @(sol) - delta*sum(sum(abs(sol))) ...
        - norm(y - A(Psi(sol)),'fro')^2/(2*sigma_noise^2);
    
    % Gradient of smooth part
    gradF_ana = @(sol) -(At(A(sol) - y))/sigma_noise^2;
    gradF_syn = @(sol) -Psit(At(A(Psi(sol)) - y))/sigma_noise^2;

    % projection to L1 norm
    soft = @(z, T) sign(z).*max(abs(z) - T*delta/2, 0);
    proxH = @(x,T) x + Psi(soft(Psit(x),T) - Psit(x));

    %% Setup the MCMC sampler
    options.samples  = 1e3;                 % number of samples to output (use 10^3 or 10^4)
    options.thinning = 1e3;                 % thinning factor (i.e., iterations per sample) (use 10^2 or 10^3)
    % This is a very important parameter. Use something of the order of 
    %     10% or 1% of the Lipchitz constant of gradF.
    options.lambda = 1e-7;                  % discretisation step (thas to be smaller than Lipschitz constante of gradF)
    options.burningNums = 1e5;              % burning numbers before recording samples

    %% Run MCMC sampler
    % Run sampler - for the analysis model
%     X0 = zeros(size(im));                   % initial condition
    X0 = At(y);
    [samPmalaAna.Xtrace, samPmalaAna.logPiTrace,samPmalaAna.acceptTrace,samPmalaAna.muTrace, samPmalaAna.time] =  ...
        PMALA(X0,logPi_ana,proxH,gradF_ana,options);
    save([pathData_ana, stringname{i}, '_samples_PMALA_ana.mat'], 'samPmalaAna');  
    
    % Run sampler - for the synthesis model
    X0 = Psit(At(y));                   % initial condition
    [samPmalaSyn.Xtrace, samPmalaSyn.logPiTrace,samPmalaSyn.acceptTrace,samPmalaSyn.muTrace,samPmalaSyn.time] =  ...
        PMALA(X0,logPi_syn,soft,gradF_syn,options);   
    save([pathData_syn, stringname{i}, '_samples_PMALA_syn.mat'], 'samPmalaSyn');    
    
    % Show results
%    figure,imagesc(squeeze(samPmalaAna.Xtrace(100,:,:))); colorbar  % - analysis model
%    figure,imagesc(Psi(squeeze(samPmalaSyn.Xtrace(100,:,:)))); colorbar  % - synthesis model
end

%% things need to follow
% 1. updata gamma in line 85 by more reasonable regularizasion parameter;
% 2. use SARA wavelets basis to update the current basis.




